#include <iostream>
#include <vector>
#define endl '\n'
using namespace std;

const int MAXN = 100000;
const int MOD = 1e8 + 543;
const int C = 1e4;

typedef vector< vector<int> > mat;

int n, q;
int a[MAXN + 5];
mat m[MAXN + 5];
mat tree[4 * MAXN + 5];
mat pw1[C + 5], pw2[C + 5];
long long lazy[4 * MAXN + 5];

mat x0 = {{0, -22, 33, -3, -4}};

mat id = {{1, 0, 0, 0, 0},
          {0, 1, 0, 0, 0},
          {0, 0, 1, 0, 0},
          {0, 0, 0, 1, 0},
          {0, 0, 0, 0, 1}};

mat mul = {{5, 1, 0, 0, 0},
           {4, 0, 1, 0, 0},
           {3, 0, 0, 1, 0},
           {2, 0, 0, 0, 1},
           {1, 0, 0, 0, 0}};

mat operator + (mat &a, mat &b)
{
    mat c;
    c.resize(a.size());
    for (int i = 0; i < a.size(); ++ i)
    {
        c[i].resize(5);
        for (int j = 0; j < 5; ++ j)
        {
            c[i][j] = (a[i][j] + b[i][j]) % MOD;
            c[i][j] = (c[i][j] + MOD) % MOD;
        }
    }
    return c;
}

mat operator * (mat &a, mat &b)
{
    mat c;
    c.resize(a.size());
    for (int i = 0; i < a.size(); ++ i)
    {
        c[i].resize(5);
        for (int j = 0; j < 5; ++ j)
        {
            c[i][j] = 0;
            for (int k = 0; k < 5; ++ k)
            {
                c[i][j] += 1ll * a[i][k] * b[k][j] % MOD;
                c[i][j] %= MOD;
                c[i][j] = (c[i][j] + MOD) % MOD;
            }
        }
    }
    return c;
}

mat comp(long long pw)
{
    long long x = pw / C, y = pw % C;
    return pw1[y] * pw2[x];
}

void build(int v, int l, int r)
{
    if (l == r)
    {
        tree[v] = m[l];
        return;
    }

    int mid = (l + r) / 2;
    build(2 * v, l, mid);
    build(2 * v + 1, mid + 1, r);
    tree[v] = tree[2 * v] + tree[2 * v + 1];
}

void push(int v, int l, int r)
{
    auto tmp = comp(lazy[v]);
    tree[v] = tree[v] * tmp;
    if (l != r)
    {
        lazy[2 * v] += lazy[v];
        lazy[2 * v + 1] += lazy[v];
    }
    lazy[v] = 0;
}

void update(int v, int l, int r, int ul, int ur, int val)
{
    push(v, l, r);
    if (ur < l or r < ul) return;
    if (ul <= l and r <= ur)
    {
        lazy[v] = val;
        push(v, l, r);
        return;
    }

    int mid = (l + r) / 2;
    update(2 * v, l, mid, ul, ur, val);
    update(2 * v + 1, mid + 1, r, ul, ur, val);
    tree[v] = tree[2 * v] + tree[2 * v + 1];
}

int query(int v, int l, int r, int ql, int qr)
{
    push(v, l, r);
    if (qr < l or r < ql) return 0;
    if (ql <= l and r <= qr) return tree[v][0][0];

    int mid = (l + r) / 2;
    int q1 = query(2 * v, l, mid, ql, qr);
    int q2 = query(2 * v + 1, mid + 1, r, ql, qr);
    return (q1 + q2) % MOD;
}

int main()
{
    ios :: sync_with_stdio(false);
    cin.tie(NULL); cout.tie(NULL);

    cin >> n >> q;

    pw1[0] = id;
    for (int i = 1; i <= C; ++ i)
    {
        pw1[i] = pw1[i - 1] * mul;
    }

    pw2[0] = id;
    for (int i = 1; i <= C; ++ i)
    {
        pw2[i] = pw2[i - 1] * pw1[C];
    }

    for (int i = 1; i <= n; ++ i)
    {
        cin >> a[i];
        auto tmp = comp(a[i]);
        m[i] = x0 * tmp;
    }

    build(1, 1, n);

    for (int i = 1; i <= q; ++ i)
    {
        int type, l, r, val;
        cin >> type >> l >> r;

        if (type == 1) cout << query(1, 1, n, l, r) << endl;
        else
        {
            cin >> val;
            update(1, 1, n, l, r, val);
        }
    }

    return 0;
}
