#include <bits/stdc++.h>
#define endl '\n'

using namespace std;

const int64_t MOD = 1e8 + 543;

const int MAXN = 1e5 + 50;
class SegmentTree
{
private:
    int64_t N, tree[4*MAXN];

    void Recalculate(int node)
    {
        tree[node] = (tree[node*2] + tree[node*2+1])%MOD;
    }

    void BuildInternal(int64_t *arr, int node, int l, int r)
    {
        if (l == r)
        {
            tree[node] = arr[l];
            return;
        }

        BuildInternal(arr, node*2, l, (l+r)/2);
        BuildInternal(arr, node*2+1, (l+r)/2+1, r);

        Recalculate(node);
    }

    void UpdateInternal(int node, int q, int64_t val, int l, int r)
    {
        if (l > q || r < q)
            return;

        if (l == r)
        {
            tree[node] = val;
            return;
        }

        UpdateInternal(node*2, q, val, l, (l+r)/2);
        UpdateInternal(node*2+1, q, val, (l+r)/2+1, r);

        Recalculate(node);
    }

    int64_t QueryInternal(int node, int ql, int qr, int l, int r)
    {
        if (r < ql || l > qr)
            return 0;

        if (ql <= l && r <= qr)
            return tree[node];

        return QueryInternal(node*2, ql, qr, l, (l+r)/2) + QueryInternal(node*2+1, ql, qr, (l+r)/2+1, r);
    }

public:
    SegmentTree()
    {
        memset(tree, 0, sizeof(tree));
    }

    SegmentTree(int N):SegmentTree()
    {
        this->N = N;
    }

    void Build(int64_t *arr)
    {
        BuildInternal(arr, 1, 0, N-1);
    }

    void Update(int q, int val)
    {
        UpdateInternal(1, q, val, 0, N-1);
    }

    int64_t Query(int ql, int qr)
    {
        return QueryInternal(1, ql, qr, 0, N-1);
    }
};

struct Matrix
{
    int64_t m[5][5];
};

void Initialize(Matrix &a)
{
    for (int i = 0; i < 5; i++)
    {
        for (int j = 0; j < 5; j++)
        {
            if (i == 0)
                a.m[i][j] = 5-j;

            else
                a.m[i][j] = (i == (j+1));
        }
    }
}
Matrix mult(Matrix &a, Matrix &b)
{
    Matrix res;
    int i, j, k;
    for (i = 0; i < 5; i++)
    {
        for (j = 0; j < 5; j++)
        {
            res.m[i][j] = 0;
            for (k = 0; k < 5; k++)
            {
                res.m[i][j] = (res.m[i][j]+a.m[i][k]*b.m[k][j]);
            }

            res.m[i][j] %= MOD;
        }
    }

    return res;
}

inline int64_t getX(int64_t ind, Matrix a)
{
    if (ind < 5)
        return ind;

    ind -= 5;

    Matrix res = a;
    while (ind)
    {
        if (ind&1)
        {
            res = mult(res, a);
        }

        ind >>= 1;
        a = mult(a, a);
    }

    int64_t x = 0;
    for (int i = 0; i < 5; i++)
        x += (res.m[0][i]* (4-i));

    x %= MOD;

    return x;
}

int64_t arr[MAXN], arrX[MAXN];
int main()
{
    ios::sync_with_stdio(false);
    cin.tie(0); cout.tie(0);

    Matrix a, b;
    Initialize(a);

    b = a;

    int N, Q;
    cin >> N >> Q;

    for (int i = 0; i < N; i++)
    {
        cin >> arr[i];

        arrX[i] = getX(arr[i], b);
    }

    SegmentTree *tree = new SegmentTree(N);
    tree->Build(arrX);

    int type, l, r;
    int64_t change, X;

    for (int i = 0; i < Q; i++)
    {

        cin >> type >> l >> r;

        if (type == 1)
        {
            if (l != r)
            {
                cout << tree->Query(l-1, r-1)%MOD << endl;
            }
            else
            {
                cout << arrX[l-1] << endl;
            }
        }

        else
        {
            cin >> change;

            arr[l-1] += change;

            X = getX(arr[l-1], b);

            arrX[l-1] = X;
            if (l != r)
                tree->Update(l-1, X);
        }
    }


    return 0;
}
