#include "puzzle.h"

#include<bits/stdc++.h>

using namespace std;

const int maxh = 51, maxn = 210;

struct edge
{
    int v, u;

    edge(int _v = 0, int _u = 0)
    {
        v = _v;
        u = _u;
    }
} edges[maxn * maxn];

int num[maxn], N, M;
double TimeLimit;
void init(int numSubtests, double _timeLimit)
{
    TimeLimit = _timeLimit;
}

int c[maxh][maxh], ridx[maxh][maxh];
pair < int, int > rev[maxh * maxh];
int h, w;

void add_edge(int v, int u)
{
    M ++;
    edges[M] = edge(v, u);
}

bool is_horizontal(edge e1)
{
    return (rev[e1.v].first == rev[e1.u].first);
}
bool cross(edge e1, edge e2)
{
    if (is_horizontal(e1) == is_horizontal(e2))
        return false;

    if (is_horizontal(e2))
        swap(e1, e2);

    int dw_row = min(rev[e2.v].first, rev[e2.u].first);
    int up_row = max(rev[e2.v].first, rev[e2.u].first);
    int lf_col = min(rev[e1.v].second, rev[e1.u].second);
    int rf_col = max(rev[e1.v].second, rev[e1.u].second);

    if (dw_row < rev[e1.v].first && up_row > rev[e1.v].first &&
        lf_col < rev[e2.v].second && rf_col > rev[e2.v].second)
            return true;

    return false;
}
void find_edges()
{
    for (int i = 0; i < h; i ++)
    {
        for (int j = 0; j < w; j ++)
        {
            if (c[i][j] == 0)
                continue;

            int r = i - 1;
            while(r >= 0 && c[r][j] == 0)
                r --;
            if (r >= 0)
            {
                add_edge(ridx[r][j], ridx[i][j]);
            }

            r = i + 1;
            while(r < h && c[r][j] == 0)
                r ++;
            if (r < h)
            {
                add_edge(ridx[r][j], ridx[i][j]);
            }

            int l = j - 1;
            while(l >= 0 && c[i][l] == 0)
                l --;
            if (l >= 0)
            {
                add_edge(ridx[i][l], ridx[i][j]);
            }

            l = j + 1;
            while(l < w && c[i][l] == 0)
                l ++;
            if (l < w)
            {
                add_edge(ridx[i][l], ridx[i][j]);
            }
        }
    }
}

vector < edge > ls;
int par[maxn];
int find_leader(int v)
{
    if (v == par[v])
        return v;
    return (par[v] = find_leader(par[v]));
}

bool unite(int v, int u)
{
    v = find_leader(v);
    u = find_leader(u);
    if (v == u)
        return false;
    par[v] = u;
    return true;
}

mt19937 rng;

void get_mst()
{

    while(true)
    {
        for (int i = 1; i <= M; i ++)
        {
            int id = rng() % M  + 1;
            swap(edges[i], edges[id]);
        }

            for (int i = 1; i <= N; i ++)
        par[i] = i;

        ls.clear();
        for (int i = 1; i <= M; i ++)
        {
            bool df = true;
            for (edge cur : ls)
            {
                if (cross(cur, edges[i]))
                    df = false;
            }
            if (!df)
                break;
            if (unite(edges[i].v, edges[i].u))
                ls.push_back(edges[i]);
        }


        if (ls.size() == N - 1)
            break;
    }

}

int path[maxn][maxn], cap[maxn];
vector < int > adj[maxn];

void build_graph()
{
    for (int i = 1; i <= N; i ++)
    {
        adj[i].clear();
    }

    for (edge cur : ls)
    {
        adj[cur.v].push_back(cur.u);
        adj[cur.u].push_back(cur.v);
    }
}

bool tf = true;
void dfs(int v, int p)
{
    int nec = 0;
    for (int u : adj[v])
    {
        if (u == p)
            continue;
        dfs(u, v);
        if (num[u] - cap[u] < 1 || num[u] - cap[u] > 2)
        {
            tf = false;
            break;
        }
        nec = nec + num[u] - cap[u];
        path[v][u] = num[u] - cap[u];
        path[u][v] = num[u] - cap[u];
    }
    if (nec > num[v])
        tf = false;
    cap[v] = nec;
    if (p == -1 && num[v] != nec)
    {
        tf = false;
    }
}


void build_path(int v, int u, int d)
{
    pair < int, int > st, en;
    st = rev[v];
    en = rev[u];

    if (st.first == en.first)
    {
        if (st.second > en.second)
            swap(st, en);

        int f = -1;
        if (d == 2)
            f =  -3;
        for (int j = st.second + 1; j < en.second; j ++)
            c[st.first][j] = f;
    }
    else
    {
        if (st.first > en.first)
            swap(st, en);
        int f = -2;
        if (d == 2)
            f = -4;
            for (int i = st.first + 1; i < en.first; i ++)
                c[i][st.second] = f;
    }
}
bool solve(vector < vector < int > > &puzzle)
{
    h = puzzle.size();
    w = puzzle[0].size();
    int cnt = 0;
    for (int i = 0; i < h; i ++)
        for (int j = 0; j < w; j ++)
        {
            if (puzzle[i][j] != 0)
            {
                ridx[i][j] = ++cnt;
                rev[cnt] = {i, j};
                N ++;
                num[N] = puzzle[i][j];
            }
            c[i][j] = puzzle[i][j];
        }

    find_edges();

    /**for (int i = 0; i < h; i ++, cout << endl)
        for (int j = 0; j < w; j ++)
        cout << setw(2) << ridx[i][j] << " ";
    for (int i = 1; i <= M; i ++)
    {
        cout << "edge " << edges[i].v << " " << edges[i].u << endl;
    }*/

    while(true)
    {
        get_mst();
        if (timePassed() > TimeLimit)
            return false;
        build_graph();
        for (int i = 1; i <= M; i ++)
        {
            path[edges[i].v][edges[i].u] = 0;
            path[edges[i].u][edges[i].v] = 0;
        }
        for (int i = 1; i <= N; i ++)
            cap[i] = 0;
        tf = true;
        dfs(1, -1);
        if (tf)
        {
            for (int i = 1; i <= N; i ++)
            {
                for (int j = i + 1; j <= N; j ++)
                {
                    if (path[i][j] > 0)
                        build_path(i, j, path[i][j]);
                }
            }

            for (int i = 0; i < h; i ++)
                for (int j = 0; j < w; j ++)
                puzzle[i][j] = c[i][j];


            return true;
        }

        ///cout << "fail" << endl;


    }

    return false;
}
