#include "puzzle.h"
#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <random>
#include <chrono>
#include <set>

const int MAXN = 200 + 10;
const int MAXRC = 50 + 10;
double perTest;

void init(int tests, double timeLimit)
{
    perTest = (timeLimit - 0.1) / (double(tests));
}

int n, m;
struct DSU
{
    int par[MAXN];
    int dep[MAXN];

    void build(int cnt)
    {
        std::iota(par + 1, par + 1 + cnt, 1);
        std::fill(dep + 1, dep + 1 + cnt, 1);
    }

    int find(int node)
    {
        if (par[node] == node) return node;
        return par[node] = find(par[node]);
    }

    void connect(int u, int v)
    {
        u = find(u);
        v = find(v);

        if (u == v)
        {
            return;
            assert(false);
        }

        if (dep[u] > dep[v])
        {
            std::swap(u, v);
        }

        if (dep[u] == dep[v])
        {
            dep[v]++;
        }

        par[u] = v;
    }

    bool areConnected(int u, int v)
    {
        return find(u) == find(v);
    }
};

int dp[MAXN][10];
std::vector <int> g[MAXN];
std::vector <int> t[MAXN];
int lastByRow[MAXRC];
int lastByCol[MAXRC];
int dpChild[MAXN][10];
int bl[MAXN][10];
int blChild[MAXN][10];
int cellRow[MAXN];
int cellCol[MAXN];
int cellIdx[MAXRC][MAXRC];
std::vector <std::pair <int,int>> edges;
int value[MAXN];
int cntNodes;
int timer;

void reset()
{
    edges.clear();
    for (int i = 0 ; i < MAXN ; ++i)
    {
        std::fill(dp[i], dp[i] + 10, 0);
        std::fill(dpChild[i], dpChild[i] + 10, 0);
        std::fill(bl[i], bl[i] + 10, 0);
        std::fill(blChild[i], blChild[i] + 10, 0);
        g[i].clear();
        t[i].clear();
    }

    for (int i = 0 ; i < MAXRC ; ++i)
    {
        std::fill(cellIdx[i], cellIdx[i] + MAXRC, 0);
    }
}

DSU dsu;

bool intersect(int fromX, int toX, int fromY, int toY)
{
    if ((cellRow[fromX] == cellRow[toX]) == (cellRow[fromY] == cellRow[toY]))
    {
        return false;
    }

    if (cellRow[fromX] != cellRow[toX])
    {
        std::swap(fromX, fromY);
        std::swap(toX, toY);
    }

    if (std::min(cellRow[fromY], cellRow[toY]) < cellRow[fromX] && cellRow[fromX] < std::max(cellRow[fromY], cellRow[toY]) && std::min(cellCol[fromX], cellCol[toX]) < cellCol[fromY] && cellCol[fromY] < std::max(cellCol[fromX], cellCol[toX]))
    {
        return true;
    }

    return false;
}

int cnt[MAXN];
std::vector <std::vector <int>> solution;
std::mt19937 rng(std::chrono::steady_clock::now().time_since_epoch().count());
bool run()
{
    dsu.build(cntNodes);
    std::vector <std::pair <int,int>> used;
    std::shuffle(edges.begin(), edges.end(), rng);
    for (const auto &[u, v] : edges)
    {
        bool isBad = false;
        for (const auto &[u2, v2] : used)
        {
            if (intersect(u2, v2, u, v))
            {
                isBad = true;
                break;
            }
        }

        if (isBad)
        {
            continue;
        }

        if (dsu.areConnected(u, v) && used.size() > 12)
        {
            continue;
        }

        used.push_back({u, v});
        dsu.connect(u, v);
    }

    for (int i = 1 ; i <= cntNodes ; ++i)
    {
        t[i].clear();
    }

    std::sort(used.begin(), used.end());
    for (int i = 0 ; i < used.size() ; ++i)
    {
        if (used[i].first > used[i].second)
        {
            std::swap(used[i].first, used[i].second);
        }
    }

    // std::cout << "set size: " << set.size() << '\n';
    // std::cout << "try\n";
    // for (const auto &[u, v] : used)
    // {
    //     std::cout << u << ' ' << v << '\n';
    // }

    for (int mask = 0 ; mask < (1 << used.size()) ; ++mask)
    {
        std::fill(cnt + 1, cnt + 1 + cntNodes, 0);
        for (int i = 0 ; i < used.size() ; ++i)
        {
            if (mask & (1 << i))
            {
                cnt[used[i].first] += 2;
                cnt[used[i].second] += 2;
            } else
            {
                cnt[used[i].first] += 1;
                cnt[used[i].second] += 1;
            }
        }

        bool isGood = true;
        for (int i = 1 ; i <= cntNodes ; ++i)
        {
            if (value[i] != cnt[i])
            {
                isGood = false;
                break;
            }
        }

        if (isGood)
        {
            for (int i = 0 ; i < used.size() ; ++i)
            {
                int value = (cellRow[used[i].first] == cellRow[used[i].second] ? -1 : -2);
                if (mask & (1 << i)) value -= 2;

                if (cellRow[used[i].first] == cellRow[used[i].second])
                {
                    for (int col = std::min(cellCol[used[i].first], cellCol[used[i].second]) + 1 ; col < std::max(cellCol[used[i].first], cellCol[used[i].second]) ; ++col)
                    {
                        solution[cellRow[used[i].first]][col] = value;
                    }
                } else
                {
                    for (int row = std::min(cellRow[used[i].first], cellRow[used[i].second]) + 1 ; row < std::max(cellRow[used[i].first], cellRow[used[i].second]) ; ++row)
                    {
                        solution[row][cellCol[used[i].first]] = value;
                    }
                }
            }

            return true;
        }
    }

    return false;
}

void getSolution(std::vector <std::vector <int>> &v)
{
    v = solution;
}

bool solve(std::vector <std::vector <int>> &v)
{
    reset();
    n = v.size();
    m = v[0].size();
    std::fill(lastByRow, lastByRow + n, -1);
    std::fill(lastByCol, lastByCol + m, -1);

    solution.clear();
    solution.resize(n);

    int idx = 0;
    for (auto &row : solution)
    {
        row.resize(m);
        for (int j = 0 ; j < m ; ++j)
        {
            row[j] = v[idx][j];
        }

        idx++;
    }

    cntNodes = 0;
    for (int i = 0 ; i < n ; ++i)
    {
        for (int j = 0 ; j < m ; ++j)
        {
            if (v[i][j] != 0)
            {
                cntNodes++;
                cellIdx[i][j] = cntNodes;
                value[cntNodes] = v[i][j];
                cellRow[cntNodes] = i; 
                cellCol[cntNodes] = j; 

                // std::cout << "number: " << i << ' ' << j << " = " << cellIdx[i][j] << '\n';

                if (lastByRow[i] != -1) 
                {
                    edges.push_back({cntNodes, cellIdx[i][lastByRow[i]]});
                    // g[cntNodes].push_back(cellIdx[i][lastByRow[i]]);
                }

                if (lastByCol[j] != -1) 
                {
                    edges.push_back({cntNodes, cellIdx[lastByCol[j]][j]});
                    // g[cntNodes].push_back(cellIdx[lastByCol[j]][j]);
                }

                lastByRow[i] = j;
                lastByCol[j] = i;
            }
        }
    }

    // std::cout << "here: " << edges.size() << ' ' << cntNodes << '\n';

    while (timePassed() < perTest)
    {
        if (run())
        {
            getSolution(v);
            return true;
        }
    }

    return false;
}