#include "puzzle.h"
#include <bits/stdc++.h>

constexpr int IMPOSSIBLE = 1;
using namespace std;

typedef pair<int, int> pii;
typedef vector<vector<int> > vvi;

struct coord {
    int x, y;
    coord() {}
    coord(int x, int y) : x(x), y(y) {}

    bool operator==(const coord& b) const {
        return x == b.x and y == b.y;
    }
};

struct Move {
    coord f, t;
    int val;
    Move() {}
    Move(coord f, coord t, int val) : f(f), t(t), val(val) {
    }
};

int num_tests, num_filled, num_islands;
double tl;
void init(int numSubtests, double timeLimit) {
    num_tests = numSubtests;
    tl = timeLimit;
}

vector<vector<vector<coord> > > graph;
vector<vector<int> > filled;

stack<Move> moves;

void print(vvi &puzzle) {
    return;
    for (int i = 0; i < puzzle.size(); i++) {
        for (int a : puzzle[i])
            cout << a << " ";
        cout << endl;
    }
    cout << endl;
    /*for (int i = 0; i < puzzle.size(); i++) {
        for (int j = 0; j < puzzle[i].size(); j++)
            cout << filled[i][j] << " ";
        cout << endl;
    }*/
    cout << num_filled << ":" << num_islands << endl;
}

vvi og_puz;
/*
@a returns bridge with val 1 if impossible
*/
Move make_bridge(coord f, coord t, int dub, vvi &puzzle) {
    if (filled[f.x][f.y] + dub + 1 > puzzle[f.x][f.y])
        return Move(f, t, IMPOSSIBLE);
    if (filled[t.x][t.y] + dub + 1 > puzzle[t.x][t.y])
        return Move(f, t, IMPOSSIBLE);
    if (f.x == t.x) {
        for (int j = min(f.y, t.y) + 1; j < max(f.y, t.y); j++) {
            if (puzzle[f.x][j] != 0)
                return Move(f, t, IMPOSSIBLE);
        }
        
        int val = HOR_SINGLE - dub*2;
        for (int j = min(f.y, t.y) + 1; j < max(f.y, t.y); j++) {
            puzzle[f.x][j] = val;
        }
        moves.push(Move(f, t, val));
        filled[f.x][f.y] += 1 + dub;
        filled[t.x][t.y] += 1 + dub;
        if (filled[f.x][f.y] == puzzle[f.x][f.y])
            num_filled++;
        if (filled[t.x][t.y] == puzzle[t.x][t.y])
            num_filled++;
        return Move(f, t, val);
    } else {
        for (int i = min(f.x, t.x) + 1; i < max(f.x, t.x); i++)
            if (puzzle[i][f.y] != 0)
                return Move(f, t, IMPOSSIBLE);
        int val = VERT_SINGLE - dub*2;
        for (int i = min(f.x, t.x) + 1; i < max(f.x, t.x); i++) {
            puzzle[i][f.y] = val;
        }
        moves.push(Move(f, t, val));
        filled[f.x][f.y] += 1 + dub;
        filled[t.x][t.y] += 1 + dub;
        if (filled[f.x][f.y] == puzzle[f.x][f.y])
            num_filled++;
        if (filled[t.x][t.y] == puzzle[t.x][t.y])
            num_filled++;
        return Move(f, t, val);
    }
}

void undo_last_move(vvi &puzzle) {
    Move me = moves.top(); moves.pop();
    coord f = me.f, t = me.t;
    if (filled[f.x][f.y] == puzzle[f.x][f.y])
        num_filled--;
    if (filled[t.x][t.y] == puzzle[t.x][t.y])
        num_filled--;
    filled[f.x][f.y]--;
    filled[t.x][t.y]--;
    if (me.val == HOR_DOUBLE || me.val == VERT_DOUBLE)
        filled[f.x][f.y]--, filled[t.x][t.y]--;
    if (f.x == t.x) {
        for (int j = min(f.y, t.y) + 1; j < max(f.y, t.y); j++)
            puzzle[f.x][j] = 0;
    } else {
        for (int i = min(f.x, t.x) + 1; i < max(f.x, t.x); i++)
            puzzle[i][f.y] = 0;
    }
}

vector<Move> get_all_moves(coord me, vvi &puzzle) {
    vector<Move> can;
    for (coord nei : graph[me.x][me.y]) {
        Move moo = make_bridge(me, nei, 0, puzzle);
        if (moo.val != IMPOSSIBLE)
        {
            can.push_back(moo);    
            undo_last_move(puzzle);
        }
        moo = make_bridge(me, nei, 1, puzzle);
        if (moo.val != IMPOSSIBLE)
        {
            can.push_back(moo); 
            undo_last_move(puzzle);
        }
    }
    return can;
}
vector<vector<Move> > get_all_sets_of_moves(coord me, vvi &puzzle) {
    vector<Move> possible = get_all_moves(me, puzzle);
    /*cout << possible.size() << endl;
    for (Move m : possible) {
        cout << m.f.x << "," << m.f.y << " " << m.t.x << "," << m.t.y << " " << m.val << endl;
    }*/
    vector<vector<Move> > out;
    for (int mask = 1; mask < (1 << possible.size()); mask++) {
        int sum = 0;
        vector<Move> curr_moves;
        for (int bit = 0; bit < possible.size(); bit++) {
            if ((1 << bit) & mask)
                curr_moves.push_back(possible[bit]);
        }
        bool good = true;
        for (int i = 0; i < curr_moves.size() - 1; i++)
        {
            if (curr_moves[i].t == curr_moves[i + 1].t)
            {
                good = false;
                break;
            }
        }
        if (!good)
            continue;
        for (Move m : curr_moves)
        {
            //if (puzzle[me.x][me.y] == 6)
            //    cout << m.t.x << " " << m.t.y << " " << m.val << endl;
            sum += 1;
            if (m.val == HOR_DOUBLE || m.val == VERT_DOUBLE)
                sum++;
        }
        
        if (sum == puzzle[me.x][me.y] - filled[me.x][me.y])
            out.push_back(curr_moves);
    }      
    return out;
}

bool recur(coord me, vvi &puzzle, bool og) {
    if (timePassed() + 0.1 > tl)
        return false;
    if (filled[me.x][me.y] == puzzle[me.x][me.y])
        return true;
    int my_number = moves.size();
    vector<vector<Move> > all_sets = get_all_sets_of_moves(me, puzzle);
    //if (puzzle[me.x][me.y] == 6)
        //cout << all_sets.size() << endl;
    // todo: for each set get affected neighbours, multiply the sizes of their sets of moves and go by that
    // right now just basic brute force
    //if (og)
        //cout << all_sets.size() << endl;
    int cnt = 0;
    for (vector<Move> my_set : all_sets) {
        cnt++;
        vector<coord> neigbs;
        for (Move m : my_set)
            neigbs.push_back(m.t);
        for (Move m : my_set) {
            int dub = 0;
            if (m.val == HOR_DOUBLE || m.val == VERT_DOUBLE)
                dub++;
            make_bridge(m.f, m.t, dub, puzzle);
        }
        print(puzzle);
        bool ok = true;
        for (coord nei : neigbs) {
            if (!recur(nei, puzzle, false)) {
                ok = false;
                break;
            }
        }
        if (!ok || (ok and og and num_islands != num_filled)) {
            if (og)
            {
                puzzle = og_puz;
                for (int i = 0; i < filled.size(); i++)
                    fill(filled[i].begin(), filled[i].end(), 0);
                num_filled = 0;
                while (!moves.empty())
                    moves.pop();
            } else {
                for (int i = 0; i < my_set.size(); i++)
                    undo_last_move(puzzle); 
            }
        } else
            return true;
        
    }
    return false;
}

bool solve(std::vector<std::vector<int>>& puzzle) {
    og_puz = puzzle;
    int n, m;
    n = puzzle.size(), m = puzzle[0].size();
    filled = vector<vector<int> > (n, vector<int> (m + 1));
    graph = vector<vector<vector<coord> > > (n, vector<vector<coord> > (m));
    // making graph;
    vector<coord> islands;
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < m; j++) {
            if (puzzle[i][j] == 0) {
                continue;
            }
            islands.push_back(coord(i, j));
            num_islands++;
            // going down
            for (int up = i - 1; up >= 0; up--) {
                if (puzzle[up][j] != 0)
                {
                    graph[i][j].push_back(coord(up, j));
                    break;
                }
            }
            // goind up
            for (int dow = i + 1; dow < n; dow++) {
                if (puzzle[dow][j] != 0) {
                    graph[i][j].push_back(coord(dow, j));
                    break;
                }
            }
            // lft
            for (int lft = j - 1; lft >= 0; lft--) {
                if (puzzle[i][lft] != 0) {
                    graph[i][j].push_back(coord(i, lft));
                    break;
                }
            }
            // rhgt
            for (int rhgt = j + 1; rhgt < m; rhgt++) {
                if (puzzle[i][rhgt] != 0) {
                    graph[i][j].push_back(coord(i, rhgt));
                    break;
                }
            }
        }
    }
    /*for (int i = 0; i < islands.size(); i++) {
        cout << i << " :";
        for (auto O : graph[islands[i].x][islands[i].y])
            cout << " " << O.x << "," << O.y;
        cout << endl;
    }*/
    //random_shuffle(islands.begin(), islands.end());
    bool ok = recur(islands[0], puzzle, true);
    return ok;
}