#include "puzzle.h"
#include <utility>
#include <iostream>
#include <bits/stdc++.h>

using namespace std;

void init(int numSubtests, double timeLimit) {
}


int n, m;
std::vector<std::vector<int>> cur_puzzle;

vector<pair<int, int>> near(int x, int y, std::vector<std::vector<int>>& puzzle) {
    vector<pair<int, int>> result;
    int k = x - 1;
    while (k >= 0 && !cur_puzzle[k][y] && puzzle[k][y] == EMPTY) k--;

    if (k >= 0 && cur_puzzle[k][y] > 0 && puzzle[k][y] > 0) {
        result.push_back(make_pair(k, y));
    }
    k = x + 1;

    while (k < n && !cur_puzzle[k][y] && puzzle[k][y] == EMPTY) k++;

    if (k < n && cur_puzzle[k][y] > 0 && puzzle[k][y] > 0) {
        result.push_back(make_pair(k, y));
    }
    k = y - 1;

    while (k >= 0 && !cur_puzzle[x][k] && puzzle[x][k] == EMPTY) k--;

    if (k >= 0 && cur_puzzle[x][k] > 0 && puzzle[x][k] > 0) {
        result.push_back(make_pair(x, k));
    }

    k = y + 1;
    while (k < m && !cur_puzzle[x][k] && puzzle[x][k] == EMPTY) k++;

    if (k < m &&  cur_puzzle[x][k] > 0 && puzzle[x][k] > 0) {
        result.push_back(make_pair(x, k));
    }

    return result;
}

std::pair<int, int> find_something(std::vector<std::vector<int>>& puzzle) {
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < m; j++) {
            if (!cur_puzzle[i][j]) {
                continue;
            }
            if (near(i, j, puzzle).size() == 1) {
                return make_pair(i, j);
            }
        }
    }

    return std::make_pair(-1, -1);
}

void connect(std::pair<int, int> pos, std::vector<std::vector<int>>& puzzle) {
    auto [x, y] = pos;

    std::pair<int, int> to_connect = near(x, y, puzzle)[0];
    int to_fill = 0;

    if (to_connect.first == x) {
        to_fill = (cur_puzzle[x][y] < 2 ? HOR_SINGLE : HOR_DOUBLE);
    } else {
        to_fill = (cur_puzzle[x][y] < 2 ? VERT_SINGLE : VERT_DOUBLE);
    }


    for (int i = std::min(to_connect.first, x) + 1; i < std::max(to_connect.first, x); i++) {
        puzzle[i][y] = to_fill;
    }
    for (int i = std::min(to_connect.second, y) + 1; i < std::max(to_connect.second, y); i++) {
        puzzle[x][i] = to_fill;
    }
    cur_puzzle[to_connect.first][to_connect.second] -= cur_puzzle[x][y];
    cur_puzzle[x][y] = 0;
}

void connect(pair<int, int> pos, pair<int, int> to_connect, int del, std::vector<std::vector<int>>& puzzle) {
    int to_fill = 0;
    auto [x, y] = pos;
    if (to_connect.first == x) {
        to_fill = (del < 2 ? HOR_SINGLE : HOR_DOUBLE);
    } else {
        to_fill = (del < 2 ? VERT_SINGLE : VERT_DOUBLE);
    }
    for (int i = std::min(to_connect.first, x) + 1; i < std::max(to_connect.first, x); i++) {
        puzzle[i][y] = to_fill;
    }
    for (int i = std::min(to_connect.second, y) + 1; i < std::max(to_connect.second, y); i++) {
        puzzle[x][i] = to_fill;
    }
    cur_puzzle[to_connect.first][to_connect.second] -= del;
    cur_puzzle[x][y] -= del;
}

void disconnect(pair<int, int> pos, pair<int, int> to_connect, int add, std::vector<std::vector<int>>& puzzle) {
    auto [x, y] = pos;
    for (int i = std::min(to_connect.first, x) + 1; i < std::max(to_connect.first, x); i++) {
        puzzle[i][y] = EMPTY;
    }
    for (int i = std::min(to_connect.second, y) + 1; i < std::max(to_connect.second, y); i++) {
        puzzle[x][i] = EMPTY;
    }
    cur_puzzle[to_connect.first][to_connect.second] += add;
    cur_puzzle[x][y] += add;
}

vector<vector<bool>> visited;
vector<pair<int, int>> edges;
set<pair<int, int>> dfs_nodes;

void count_edges(int x, int y, pair<int, int> cur_d, std::vector<std::vector<int>>& puzzle) {
    if (x < 0 || y < 0 || x >= n || y >= m || visited[x][y] || puzzle[x][y] == EMPTY) {
        return;
    }

    visited[x][y] = true;
    if (puzzle[x][y] > 0) {
        dfs_nodes.insert(make_pair(x, y));
        vector<pair<int, int>> cur_edges = near(x, y, puzzle);
        for (auto lol : cur_edges) {
            edges.push_back(lol);
        }
        count_edges(x - 1, y, {-1, 0}, puzzle);
        count_edges(x + 1, y, {1, 0}, puzzle);
        count_edges(x, y - 1, {0, -1}, puzzle);
        count_edges(x, y + 1, {0, 1}, puzzle);
    } else {
        auto [dx, dy] = cur_d;
        count_edges(x + dx, y + dy, cur_d, puzzle);
    }


}

bool norm(int lx, int ly, int rx, int ry, std::vector<std::vector<int>>& puzzle) {
    int to_del = 1;
    connect(make_pair(lx, ly), make_pair(rx, ry), to_del, puzzle);


    bool flag = true;

    for (int i = 0; i < n && flag; i++) {
        for (int j = 0; j < m; j++) {
            if (!cur_puzzle[i][j]) continue;

            if ((i == lx && j == ly) || (i == rx && j == ry)) continue;

            int min_near = (cur_puzzle[i][j] + 1) / 2;

            if (near(i, j, puzzle).size() < min_near) {
                flag = false;
                break;
            }
        }
    }

    visited = vector<vector<bool>>(n, vector<bool>(m, false));
    edges.clear();
    dfs_nodes.clear();
    count_edges(lx, ly, {9, 0}, puzzle);

    int ans = 0;
    for (auto x : edges) {
        ans += !dfs_nodes.count(x);
    }

    flag &= ans > 0;


    disconnect(make_pair(lx, ly), make_pair(rx, ry), to_del, puzzle);

    return flag;
}

bool solve(std::vector<std::vector<int>>& puzzle) {
    n = puzzle.size();
    m = puzzle[0].size();
    cur_puzzle = puzzle;

    while (true) {
        std::pair<int, int> current = find_something(puzzle);
        while (current.first != -1) {
            //cout << "INSIDE: " << current.first << ' ' << current.second << endl;
            connect(current, puzzle);

            current = find_something(puzzle);
        }
        bool updated = false;

        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                if (!cur_puzzle[i][j]) continue;

                vector<pair<int, int>> possible, currently = near(i, j, puzzle);

                for (auto [x, y] : currently) {
                    if (norm(i, j, x, y, puzzle)) {
                        possible.push_back(make_pair(x, y));
                    }
                }

                if (possible.size() == 1) {
                    assert(cur_puzzle[i][j] <= 2 && cur_puzzle[possible[0].first][possible[0].second] >= cur_puzzle[i][j]);
                    updated = true;
                    connect(make_pair(i, j), possible[0], cur_puzzle[i][j], puzzle);
                }
            }
        }

        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                if (!cur_puzzle[i][j]) continue;

                vector<pair<int, int>> currently = near(i, j, puzzle);

                if (cur_puzzle[i][j] / 2 == currently.size()) {
                    for (auto [x, y] : currently) {
                        assert(cur_puzzle[x][y] >= 2);
                        connect(make_pair(i, j), make_pair(x, y), min(2, cur_puzzle[x][y]), puzzle);
                        updated = true;
                    }
                } else if ((cur_puzzle[i][j] + 1) / 2 == currently.size()) {
                    sort(currently.begin(), currently.end(), [&](auto x, auto y) {
                        return cur_puzzle[x.first][x.second] > cur_puzzle[y.first][y.second];
                    });

                    int left = cur_puzzle[i][j];
                    for (auto [x, y] : currently) {
                        connect(make_pair(i, j), make_pair(x, y), min(2, left), puzzle);
                        left = max(0, left - 2);
                    }
                }

            }
        }

        if (!updated) {
            // cout << "FUCK<3" << endl;
            break;
        }
        // cout << "[+] Something found!" << endl;
    }
    /*cout << "AFTER connect: " << endl;
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < m; j++) {
            cout << cur_puzzle[i][j] << ' ';
        }
        cout << endl;
    }*/

    /*for (int i = 0; i < n; i++) {
        for (int j = 0; j < m; j++) {
            cout << puzzle[i][j] << ' ';
        }
        cout << endl;
    }*/

    return true;
}
