#include "park.h"
#include <random>
#include <iostream>
#include <algorithm>
#include <map>

using namespace std;

std::mt19937 rnd(std::random_device{}());

double timelimit;

double pick_double() {
	std::uniform_real_distribution<double> pick(0.0, 1.0);
	return pick(rnd);
}

int pick_int(int mn, int mx) {
	uniform_int_distribution<int> pick(mn, mx);
	return pick(rnd);
}

void init(int numSubtests, double timeLimit) {
	timelimit = timeLimit;
}

pair<int, int> getUp(std::vector<std::vector<int>>& puzzle, int x, int y) {
	if (x <= 0)
		return { -1, -1 };
	if (puzzle[x - 1][y] != 0 && puzzle[x - 1][y] != -3 && puzzle[x - 1][y] != -4) {
		return { -1, -1 };
	}
	for (int i = x - 1; i >= 0; --i) {
		if (puzzle[i][y] != 0 && puzzle[i][y] != -3 && puzzle[i][y] != -4) {
			return { i, y };
		}
	}
	return { -1, -1 };
}

pair<int, int> getDown(std::vector<std::vector<int>>& puzzle, int x, int y) {
	if (x >= puzzle.size() - 1)
		return { -1, -1 };
	if (puzzle[x + 1][y] != 0 && puzzle[x + 1][y] != -3 && puzzle[x + 1][y] != -4) {
		return { -1, -1 };
	}
	for (int i = x + 1; i < puzzle.size(); ++i) {
		if (puzzle[i][y] != 0 && puzzle[i][y] != -3 && puzzle[i][y] != -4) {
			return { i, y };
		}
	}
	return { -1, -1 };
}

pair<int, int> getLeft(std::vector<std::vector<int>>& puzzle, int x, int y) {
	if (y <= 0)
		return { -1, -1 };
	if (puzzle[x][y - 1] != 0 && puzzle[x][y - 1] != -1 && puzzle[x][y - 1] != -2) {
		return { -1, -1 };
	}
	for (int i = y - 1; i >= 0; --i) {
		if (puzzle[x][i] != 0 && puzzle[x][i] != -1 && puzzle[x][i] != -2) {
			return { x, i };
		}
	}
	return { -1, -1 };
}

pair<int, int> getRight(std::vector<std::vector<int>>& puzzle, int x, int y) {
	if (y >= puzzle[0].size() - 1)
		return { -1, -1 };
	if (puzzle[x][y + 1] != 0 && puzzle[x][y + 1] != -1 && puzzle[x][y + 1] != -2) {
		return { -1, -1 };
	}
	for (int i = y + 1; i < puzzle[0].size(); ++i) {
		if (puzzle[x][i] != 0 && puzzle[x][i] != -1 && puzzle[x][i] != -2) {
			return { x, i };
		}
	}
	return { -1, -1 };
}

bool canEdit(std::vector<std::vector<int>>& puzzle, int x1, int y1, int x2, int y2) {
	if (x1 != x2 && y1 != y2) {
		return 0;
	}
	if (x1 == x2 && y1 == y2) {
		return 0;
	}
	if (x1 != x2) {
		if (x1 > x2)
			swap(x1, x2);
		if (getDown(puzzle, x1, y1) != make_pair(x2, y2))
			return 0;
	}
	if (y1 != y2) {
		if (y1 > y2)
			swap(y1, y2);
		if (getRight(puzzle, x1, y1) != make_pair(x2, y2))
			return 0;
	}
	return 1;
}

bool hasAir(std::vector<std::vector<int>>& puzzle, int x1, int y1, int x2, int y2) {
	if (x1 != x2 && y1 != y2) {
		return 0;
	}
	if (x1 == x2 && y1 == y2) {
		return 0;
	}
	if (x1 != x2) {
		if (x1 > x2)
			swap(x1, x2);
		if (getDown(puzzle, x1, y1) != make_pair(x2, y2))
			return 0;
		if (puzzle[x1 + 1][y1] == -4)
			return 0;
	}
	if (y1 != y2) {
		if (y1 > y2)
			swap(y1, y2);
		if (getRight(puzzle, x1, y1) != make_pair(x2, y2))
			return 0;
		if (puzzle[x1][y1 + 1] == -2)
			return 0;
	}
	return 1;
}

bool hasBridge(std::vector<std::vector<int>>& puzzle, int x1, int y1, int x2, int y2) {
	if (x1 != x2 && y1 != y2) {
		return 0;
	}
	if (x1 == x2 && y1 == y2) {
		return 0;
	}
	if (x1 != x2) {
		if (x1 > x2)
			swap(x1, x2);
		if (getDown(puzzle, x1, y1) != make_pair(x2, y2))
			return 0;
		if (puzzle[x1 + 1][y1] == 0)
			return 0;
	}
	if (y1 != y2) {
		if (y1 > y2)
			swap(y1, y2);
		if (getRight(puzzle, x1, y1) != make_pair(x2, y2))
			return 0;
		if (puzzle[x1][y1 + 1] == 0)
			return 0;
	}
	return 1;
}

void editBridge(std::vector<std::vector<int>>& puzzle, int x1, int y1, int x2, int y2, bool destr) {
	if (x1 != x2) {
		if (destr) {
			for (int i = min(x1, x2) + 1; i < max(x1, x2); ++i) {
				puzzle[i][y1]++;
				if (puzzle[i][y1] == -2)
					puzzle[i][y1] = 0;
			}
		}
		else {
			for (int i = min(x1, x2) + 1; i < max(x1, x2); ++i) {
				if (puzzle[i][y1] == 0)
					puzzle[i][y1] = -2;
				puzzle[i][y1]--;
			}
		}
	}
	if (y1 != y2) {
		if (destr) {
			for (int i = min(x1, x2) + 1; i < max(x1, x2); ++i) {
				puzzle[i][y1]++;
			}
		}
		else {
			for (int i = min(x1, x2) + 1; i < max(x1, x2); ++i) {
				puzzle[i][y1]--;
			}
		}
	}
}

bool isBuildRight(std::vector<std::vector<int>>& puzzle, map<int, map<int, int>>& cnt, int x1, int y1, int x2, int y2) {
	if (hasAir(puzzle, x1, y1, x2, y2) && cnt[x1][y1] < puzzle[x1][y1] && cnt[x2][y2] < puzzle[x2][y2])
		return 1;
	return 0;
}

bool isDestroyRight(std::vector<std::vector<int>>& puzzle, map<int, map<int, int>>& cnt, int x1, int y1, int x2, int y2) {
	if (hasBridge(puzzle, x1, y1, x2, y2) && cnt[x1][y1] > puzzle[x1][y1] && cnt[x2][y2] > puzzle[x2][y2])
		return 1;
	return 0;
}

bool solved(std::vector<std::vector<int>>& puzzle, map<int, map<int, int>>& cnt) {
	for (auto& [x1, mp2] : cnt) {
		for (auto& [y1, cnt2] : mp2) {
			if (puzzle[x1][y1] != cnt2) {
				return false;
			}
		}
	}
	return true;
}

void makeDecition(std::vector<std::vector<int>>& puzzle, map<int, map<int, int>>& cnt, int x1, int y1, int x2, int y2, double& temp) {
	bool right = 0;
	if (!canEdit(puzzle, x1, y1, x2, y2)) {

	}
	bool destr = 0;
	if (isBuildRight(puzzle, cnt, x1, y1, x2, y2)) {
		destr = 0;
		right = 1;
	} else if (isDestroyRight(puzzle, cnt, x1, y1, x2, y2)) {
		destr = 1;
		right = 1;
	} else if (hasAir(puzzle, x1, y1, x2, y2)) {
		destr = 0;
		right = 0;
	} else {
		destr = 1;
		right = 0;
	}
	if (right) {
		if (destr) {
			cnt[x1][y1]--;
			cnt[x2][y2]--;
		} else {
			cnt[x1][y1]++;
			cnt[x2][y2]++;
		}
		editBridge(puzzle, x1, y1, x2, y2, destr);
	} else {
		double ch = pick_double();
		if (ch < temp) {
			temp *= 0.9;
			if (destr) {
				cnt[x1][y1]--;
				cnt[x2][y2]--;
			} else {
				cnt[x1][y1]++;
				cnt[x2][y2]++;
			}
			editBridge(puzzle, x1, y1, x2, y2, destr);
		}
	}
}

bool solve(std::vector<std::vector<int>>& puzzle) {
	if (timePassed() + 0.3 >= timelimit)
		return false;

	double temp = 1.0;
	vector<pair<int, int> > islands;
	map<int, map<int, vector<pair<int, int> > > > connected;
	map<int, map<int, int> > cnt;

	for (int i = 0; i < puzzle.size(); ++i) {
		for (int j = 0; j < puzzle[i].size(); ++j) {
			if (puzzle[i][j] > 0) {
				islands.push_back({ i, j });
				cnt[i][j] = 0;
				if (getUp(puzzle, i, j) != make_pair(-1, -1)) {
					connected[i][j].push_back(getUp(puzzle, i, j));
				}
				if (getDown(puzzle, i, j) != make_pair(-1, -1)) {
					connected[i][j].push_back(getDown(puzzle, i, j));
				}
				if (getRight(puzzle, i, j) != make_pair(-1, -1)) {
					connected[i][j].push_back(getRight(puzzle, i, j));
				}
				if (getLeft(puzzle, i, j) != make_pair(-1, -1)) {
					connected[i][j].push_back(getLeft(puzzle, i, j));
				}
			}
		}
	}

	while (!solved(puzzle, cnt)) {
		if (timePassed() + 0.3 >= timelimit)
			return false;
		auto [x1, y1] = islands[pick_int(0, islands.size() - 1)];
		if (connected[x1][y1].size() == 0)
			return false;
		auto [x2, y2] = connected[x1][y1][pick_int(0, connected[x1][y1].size() - 1)];
		makeDecition(puzzle, cnt, x1, y1, x2, y2, temp);
	}

	return true;
}