#include <iostream>
#include <vector>
using namespace std;

const int M = 1e8 + 543;
typedef vector<vector<int>> mx;
typedef int64_t ll;

mx operator+(const mx& m1, const mx &m2) {
    vector<vector<int>> res(5, vector<int>(5));
    for (int i = 0; i < 5; ++i) {
        for (int j = 0; j < 5; ++j) {
            res[i][j] = ((ll)m1[i][j] + m2[i][j])% M;
        }
    }
    return res;
}
mx operator*(const mx& m1, const mx &m2) {
    vector<vector<int>> res(5, vector<int>(5));
    for (int i = 0; i < 5; ++i) {
        for (int j = 0; j < 5; ++j) {
            for (int k = 0; k < 5; ++k) {
                res[i][j] += ((ll)m1[i][k]*m2[k][j]) % M;
            }
        }
    }
    return res;
}

vector<mx> v(32);
const mx mat = {{5,4,3,2,1},{1,0,0,0,0},{0,1,0,0,0},{0,0,1,0,0}, {0,0,0,1,0}};

void init() {
    v[0] = mat;

    for (int i = 1; i < 32; ++i)
        v[i] = v[i - 1]*v[i - 1];

}

mx exp(const mx &a, int b) {
    if (b == 1) return a;

    mx tmp = exp(a, b/2);
    tmp = tmp*tmp;

    if (b%2) return tmp*a;

    return tmp;
}
mx exp2(int b) {
    mx res;
    for (int j = 0; j < 32 - __builtin_clz(b); ++j)
        if ((b >> j)&1)
            res = res + v[j];
    return res;
}

int x(int i) {
    if (i < 5) return i;

    mx m = exp(mat, i - 4);
    int res = 0;

    for (int k = 0; k < 5; ++k)
        res = (res + ((ll)(5 - k - 1)*m[0][k])%M)%M;

    return res;
}

class ST {
public:
    explicit ST(const vector<int> &arr) {
        int n = arr.size();
        N = 1 << (32 - __builtin_clz(n - 1));
        t = vector<int>(2*N);
        lazy = vector<int>(2*N);

        for (int i = 0; i < n; ++i)
            t[i + N] = arr[i];

        for (int i = N - 1; i; --i)
            t[i] = t[2*i] + t[2*i + 1];
    }

    void push(int pos) {
        if (lazy[pos] == 0 || pos >= N) return;

        t[pos*2] += lazy[pos];
        lazy[pos*2] += lazy[pos];
        t[pos*2 + 1] += lazy[pos];
        lazy[pos*2 + 1] += lazy[pos];

        lazy[pos] = 0;
    }

    void update(int pos, int l1, int r1, int l, int r, int val) {
        if (l1 == l && r1 == r) {
            t[pos] = (t[pos] + val)%M;
            lazy[pos] = (lazy[pos] + val)%M;
            return;
        }

        push(pos);
        update(pos*2, l1, (r1 + l1)/2, l, min(r, (r1 + l1)/2), val);
        update(pos*2+1, (r1 + l1)/2 + 1,  r1, max((r1 + l1)/2 + 1, l), r, val);
    }

    int query(int pos, int l1, int r1, int l, int r) {
        if (r1 < l || l1 > r) return 0;
        if (l1 == l && r1 == r) return t[pos];

        push(pos);

        return (query(pos*2, l1, (r1 + l1)/2, l, min(r, (r1 + l1)/2))
        + query(pos*2+1, (r1 + l1)/2 + 1,  r1, max((r1 + l1)/2 + 1, l), r)) % M;
    }

private:
    int N;
    vector<int> t, lazy;
};

int main() {
    init();

    int n, q; cin >> n >> q;
    vector<int> a(n);
    for (int &i : a) cin >> i;

    vector<int> z(n);
    for (int i = 0; i < n; ++i) z[i] = x(a[i]);

    int l, r, type, val;
    while(q--) {
        cin >> type;
        if (type == 1) {
            cin >> l >> r;
            cout << z[l - 1];
        } else {
            cin >> l >> r >> val;
            a[l - 1] += val;
            z[l - 1] = x(a[l - 1]);
        }
    }


/*
    ST tree(z);



    int l, r, type, val;
    while(q--) {
        cin >> type;

        if (type == 1) {
            cin >> l >> r;
            cout << tree.query(1, 0, n - 1, l - 1, r - 1) << '\n';
        } else {
            cin >> l >> r >> val;
            tree.update(1, 0, n - 1, l - 1, r - 1, val);
        }
    }
    */
}