#include <bits/stdc++.h>

using namespace std;

constexpr int MOD = 1e8 + 543;
constexpr int NMAX = 1e5 + 5;
typedef long long LL;

int Modulo (LL x) {
    int ab = abs(x) % MOD;

    if (x < 0) return MOD - ab;

    return ab;
}

struct Matrix {
    int l, c;

    vector <vector <int> > values;

    Matrix (int n, int m) {
        values.resize(n);
        for (int i = 0; i < n; ++ i )
            values[i].resize(m, 0);
        l = n;
        c = m;
    }

    Matrix () {
        l = 0;
        c = 0;
    }

    Matrix operator * (Matrix other) {
        Matrix answer(l, other.c);

        if (c != other.l) {
            cout << "Impossible\n";
            return answer;
        }

        for (int i = 0; i < answer.l; ++ i ) {
            for (int j = 0; j < answer.c; ++ j ) {
                for (int k = 0; k < c; ++ k ) {
                    answer.values[i][j] = Modulo(1LL * answer.values[i][j] + 1LL * Modulo(1LL * values[i][k] * other.values[k][j]));
                }
            }
        }

        return answer;
    }

    void operator *= (Matrix other) {
        *this = (*this) * other;
    }

    Matrix operator + (Matrix other) {
        Matrix answer(l, c);

        if (l != other.l || c != other.c) {
            cout << "Impossible\n";
            return answer;
        }

        for (int i = 0; i < answer.l; ++ i ) {
            for (int j = 0; j < answer.c; ++ j ) {
                answer.values[i][j] = Modulo(1LL * values[i][j] + 1LL * other.values[i][j]);
            }
        }

        return answer;
    }

    bool operator == (Matrix other) {
        if (l != other.l || c != other.c) return false;

        for (int i = 0; i < l; ++ i )
            for (int j = 0; j < c; ++ j )
                if (values[i][j] != other.values[i][j]) return false;

        return true;
    }
};

Matrix normal(5, 5), invers(5, 5);
Matrix Ident(5, 5);

int N, Q;
int A[NMAX];

Matrix LgPut (Matrix a, int b) {
    Matrix ans = Ident;

    while (b != 0) {
        if (b % 2 == 0) {
            b /= 2;
            a *= a;
        }
        else {
            -- b;
            ans *= a;
        }
    }

    return ans;
}

vector <Matrix> aint;
vector <Matrix> Lazy;

void Add (int nod, Matrix who) {
    Lazy[nod] *= who;
    aint[nod] *= who;
}

void Propag (int nod, int a, int b) {
    if (a == b) return;

    for (int i = 2 * nod; i <= 2 * nod + 1; ++ i )
        Add(i, Lazy[nod]);

    Lazy[nod] = Ident;
}

void Init (int nod, int a, int b) {
    if (a == b) {
        if (A[a] < 0) aint[nod] = LgPut(invers, -A[a]);
        else aint[nod] = LgPut(normal, A[a]);

        Lazy[nod] = Ident;
        return;
    }

    int mij = (a + b) / 2;

    Init(2*nod, a, mij);
    Init(2*nod+1, mij+1, b);

    aint[nod] = aint[2*nod] + aint[2*nod+1];
    Lazy[nod] = Ident;
}

void Update (int nod, int a, int b, int ua, int ub, Matrix who) {
    if (ua <= a && b <= ub) {
        Add(nod, who);
        return;
    }

    Propag(nod, a, b);
    int mij = (a + b) / 2;

    if (ua <= mij) Update(2*nod, a, mij, ua, ub, who);
    if (mij < ub) Update(2*nod+1, mij+1, b, ua, ub, who);

    aint[nod] = aint[2*nod] + aint[2*nod+1];
}

Matrix Query (int nod, int a, int b, int qa, int qb) {
    if (qa <= a && b <= qb) return aint[nod];

    Propag(nod, a, b);
    int mij = (a + b) / 2;
    Matrix st(5, 5), dr(5, 5);

    if (qa <= mij) st = Query(2*nod, a, mij, qa, qb);
    if (mij < qb) dr = Query(2*nod+1, mij+1, b, qa, qb);

    return st + dr;
}

void Read () {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    cin >> N >> Q;

    for (int i = 1; i <= N; ++ i ) {
        cin >> A[i];
        A[i] -= 4;
    }

    aint.resize(8 * N);
    Lazy.resize(8 * N);
    Init(1, 1, N);
}

void Precompute () {
    for (int i = 0; i < 5; ++ i )
        Ident.values[i][i] = 1;

    normal.values[1][0] = normal.values[2][1] = normal.values[3][2] = normal.values[4][3] = 1;
    for (int i = 0; i < 5; ++ i )
        normal.values[i][4] = i + 1;
    invers.values[3][4] = invers.values[2][3] = invers.values[1][2] = invers.values[0][1] = 1;
    invers.values[4][0] = 1;
    for (int i = 0; i < 4; ++ i )
        invers.values[i][0] = -i - 2;

    assert(invers * normal == Ident);
}

void Solve () {
    Matrix sir(1, 5);
    for (int i = 0; i < 5; ++ i )
        sir.values[0][i] = i;

    for (; Q; -- Q ) {
        int type, l, r;

        cin >> type >> l >> r;

        if (type == 1) {
            Matrix v = sir * Query(1, 1, N, l, r);

            cout << v.values[0][4] << '\n';
        }
        else {
            int v;
            cin >> v;

            Update(1, 1, N, l, r, (v < 0 ? LgPut(invers, -v) : LgPut(normal, v)));
        }
    }
}

int main()
{
    Precompute();
    Read();
    Solve();

    return 0;
}
