#include "ones.h"
#include <iostream>
#include <random>
#include <chrono>
#include <algorithm>
#include <cmath>

using namespace std;

mt19937 rng(chrono::steady_clock::now().time_since_epoch().count());

double randInRange(double a, double b, double power = 1.0)
{
    double amogus = ((double)(rng()) / (double)(0xffffffff));
    return a + pow(amogus, power) * (b-a);
}

const double bulanMinMult = 0.5d;
const double bulanMaxMult = 0.9d;
const double bulanMultPow = 1.1;
const double bulanMinAdd = 1.0;
const double bulanMaxAdd = 1.0;

const double binMinBias = 0.35;
const double binMaxBias = 0.65;
const double binBiasPow = 1.2;
const double binMinAdd = -1.0;
const double binMaxAdd = 0.5;

int onePozIndex;
vector<int> onePoz;

void initOnePoz(int n, int k)
{
    onePoz.clear();
    for (int i = 1; i <= n; i++) {
        onePoz.push_back(i);
    }
    for (int i = 0; i < onePoz.size(); i++) {
        int poz = rng() % (i+1);
        swap(onePoz[i], onePoz[poz]);
    }
    while (onePoz.size() > k) onePoz.pop_back();
    sort(onePoz.begin(), onePoz.end());
    for (int i = onePoz.size()-1; i >= 1; i--) {
        onePoz[i] = onePoz[i] - onePoz[i-1];
    }
    onePozIndex = 0;
}

int getFunnyNumber(int n, int k)
{
    double amogus = ((double)(n)) / k;
    double funny = amogus * randInRange(bulanMinMult, bulanMaxMult, bulanMultPow) + randInRange(bulanMinAdd, bulanMaxAdd);
    return max(1, int(funny));
    /*int dif = onePoz[onePozIndex];
    onePozIndex = (onePozIndex+1) % onePoz.size();
    int amogus = max(1, (int)(dif * randInRange(bulanMinMult, bulanMaxMult) + randInRange(bulanMinAdd, bulanMaxAdd)));
    return amogus;*/
}

int getBinSearchMid(int st, int dr)
{
    double absoluteMid = ((double)(st+dr)) / 2.0d;
    double fuzzyMid = st+1 + (dr-st-2) * randInRange(binMinBias, binMaxBias, binBiasPow) + randInRange(binMinAdd, binMaxAdd);
    return max(st+1, min(dr-1, int(fuzzyMid)));
}

int binSearch(int st, int dr)
{
    /// ( ]
    int mij;
    while (dr-st > 1) {
        mij = getBinSearchMid(st, dr);
        if (hasOnes(st+1, mij)) {
            dr = mij;
        }
        else {
            st = mij;
        }
    }
    return dr;
}

vector<int> guessOnes(int n, int k)
{
    initOnePoz(n, k);
    vector<int> ans;
    int st = 0;
    while (k) {
        int dr = min(n, st + getFunnyNumber(n-st, k));
        while (!hasOnes(st+1, dr)) {
            st = dr;
            dr = min(n, st + getFunnyNumber(n-st, k));
        }
        ans.push_back(binSearch(st, dr));
        st = ans.back();
        k--;
    }
    return ans;
}
