#include "puzzle.h"
#include <random>
#include <chrono>

using namespace std;

mt19937 rng(chrono::steady_clock::now().time_since_epoch().count());

int numSubtests;
double timeLimit;
int n, m;
int p[] = {-1, -2, -3, -4};
int dx[] = {-1, 0, 1, 0};
int dy[] = {0, 1, 0, -1};
pair<int, int> nb[4][50][50];
int deg[50][50];

void init(int a, double b)
{
    numSubtests = a;
    timeLimit = b;
}

void calcNb(vector<vector<int>> &puzzle)
{
    pair<int, int> aux;
    for (int i = 0; i < n; i++) {
        aux = {-1, -1};
        for (int j = 0; j < m; j++) {
            nb[3][i][j] = aux;
            if (puzzle[i][j]) {
                aux = {i, j};
            }
        }
        aux = {-1, -1};
        for (int j = m-1; j >= 0; j--) {
            nb[1][i][j] = aux;
            if (puzzle[i][j]) {
                aux = {i, j};
            }
        }
    }
    for (int j = 0; j < m; j++) {
        aux = {-1, -1};
        for (int i = 0; i < n; i++) {
            nb[0][i][j] = aux;
            if (puzzle[i][j]) {
                aux = {i, j};
            }
        }
        aux = {-1, -1};
        for (int i = n-1; i >= 0; i--) {
            nb[2][i][j] = aux;
            if (puzzle[i][j]) {
                aux = {i, j};
            }
        }
    }
}

bool tryEdge(vector<vector<int>> &puzzle, int x, int y, int id)
{
    int cost;
    if (id == -1 || id == -2) cost = 1;
    else cost = 2;
    if (deg[x][y]+cost > puzzle[x][y]) return false;
    bool vert;
    if (id == -2 || id == -4) vert = true;
    else vert = false;
    if (vert) {
        int x2 = nb[2][x][y].first;
        int y2 = nb[2][x][y].second;
        if (x2 == -1 || y2 == -1) return false;
        if (deg[x2][y2]+cost > puzzle[x2][y2]) return false;
        for (int i = x+1; i <= x2-1; i++) {
            if (puzzle[i][y] != 0) return false;
        }
        for (int i = x+1; i <= x2-1; i++) {
            puzzle[i][y] = id;
        }
        deg[x][y] += cost;
        deg[x2][y2] += cost;
        return true;
    }
    int x2 = nb[1][x][y].first;
    int y2 = nb[1][x][y].second;
    if (x2 == -1 || y2 == -1) return false;
    if (deg[x2][y2]+cost > puzzle[x2][y2]) return false;
    for (int j = y+1; j <= y2-1; j++) {
        if (puzzle[x][j] != 0) return false;
    }
    for (int j = y+1; j <= y2-1; j++) {
        puzzle[x][j] = id;
    }
    deg[x][y] += cost;
    deg[x2][y2] += cost;
    return true;
}

bool addEdges(vector<vector<int>> &puzzle)
{
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < m; j++) {
            deg[i][j] = 0;
            if (puzzle[i][j] < 0) puzzle[i][j] = 0;
        }
    }
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < m; j++) {
            if (puzzle[i][j] <= 0) continue;
            for (int i = 0; i < 4; i++) {
                int poz = rng() % (i+1);
                swap(p[i], p[poz]);
            }
            for (int i = 0; i < 4; i++) {
                tryEdge(puzzle, i, j, p[i]);
            }
            if (deg[i][j] < puzzle[i][j]) return false;
        }
    }
    return true;
}

bool solve(vector<vector<int>> &puzzle)
{
    n = puzzle.size();
    m = puzzle[0].size();
    calcNb(puzzle);
    while (!addEdges(puzzle)) {
        if (timePassed() >= 8.0) return false;
    }
    return true;
}
