#include <iostream>
#include <iomanip>

using namespace std;

#define MOD 100000543

int n, n2, q, i, a[100010];

struct Query
{
    int tip, l, r, val;
} currq;

struct Matrix
{
    int a[25];
    void zero()
    {
        for (int i = 0; i < 25; i++) {
            a[i] = 0;
        }
    }
    void identity()
    {
        zero();
        a[0] = a[6] = a[12] = a[18] = a[24] = 1;
    }
    static void cpy(Matrix &dst, const Matrix &src)
    {
        for (int i = 0; i < 25; i++) {
            dst.a[i] = src.a[i];
        }
    }
    static void mult(const Matrix &a, const Matrix &b, Matrix &c)
    {
        for (int i = 0; i < 5; i++) {
            for (int j = 0; j < 5; j++) {
                #define F(k) a.a[i*5+k] * b.a[k*5+j]
                c.a[i*5+j] = (F(0) + F(1) + F(2) + F(3) + F(4)) % MOD;
                #undef F
            }
        }
    }
    static void mult(const Matrix &a, int *v, int *w)
    {
        for (int i = 0; i < 5; i++) {
            #define F(k) a.a[i*5+k] * v[k]
            w[i] = (F(0) + F(1) + F(2) + F(3) + F(4)) % MOD;
        }
    }
};

Matrix binMatUp[30], binMatDown[30];
void calcBinMats()
{
    binMatUp[0] = {
    0, 1, 0, 0, 0,
    0, 0, 1, 0, 0,
    0, 0, 0, 1, 0,
    0, 0, 0, 0, 1,
    1, 2, 3, 4, 5
    };
    binMatDown[0] = {
    -2, -3, -4, -5, 1,
    1, 0, 0, 0, 0,
    0, 1, 0, 0, 0,
    0, 0, 1, 0, 0,
    0, 0, 0, 1, 0
    };
    for (int i = 1; (1 << i) <= MOD; i++) {
        Matrix::mult(binMatUp[i-1], binMatUp[i-1], binMatUp[i]);
        Matrix::mult(binMatDown[i-1], binMatDown[i-1], binMatDown[i]);
    }
}
void getRecurrenceMatrix(int step, Matrix &ans, Matrix &aux)
{
    ans.identity();
    if (step >= 0) {
        for (int i = 0; (1 << i) <= step; i++) {
            if (step & (1 << i)) {
                Matrix::mult(ans, binMatUp[i], aux);
                Matrix::cpy(ans, aux);
            }
        }
    }
    else {
        step = -step;
        for (int i = 0; (1 << i) <= step; i++) {
            if (step & (1 << i)) {
                Matrix::mult(ans, binMatDown[i], aux);
                Matrix::cpy(ans, aux);
            }
        }
    }
}

struct Aint
{
    struct Data
    {
        int s[5];
        Matrix lazy;
    };
    Data v[262152];
    int auxS[5];
    Matrix auxMat, qMat;
    int ql, qr;
    void propag(int node)
    {
        Matrix::mult(v[node].lazy, v[node].s, auxS);
        for (int i = 0; i < 5; i++) {
            v[node].s[i] = auxS[i];
        }
        if (node < n2) {
            Matrix::mult(v[node].lazy, v[2*node].lazy, auxMat);
            Matrix::cpy(v[2*node].lazy, auxMat);
            Matrix::mult(v[node].lazy, v[2*node+1].lazy, auxMat);
            Matrix::cpy(v[2*node+1].lazy, auxMat);
        }
        v[node].lazy.identity();
    }
    void init()
    {
        for (int i = n2; i < 2*n2; i++) {
            for (int j = 0; j < 5; j++) {
                v[i].s[j] = j;
            }
            v[i].lazy.identity();
        }
        for (int i = 1; i <= n; i++) {
            getRecurrenceMatrix(a[i], v[n2+i-1].lazy, auxMat);
            propag(n2+i-1);
        }
        for (int i = n2-1; i >= 1; i--) {
            v[i].lazy.identity();
            for (int j = 0; j < 5; j++) {
                v[i].s[j] = (v[2*i].s[j] + v[2*i+1].s[j]) % MOD;
            }
        }
    }
    void update(int node, int l, int r)
    {
        if (ql <= l && r <= qr) {
            Matrix::mult(v[node].lazy, qMat, auxMat);
            Matrix::cpy(v[node].lazy, auxMat);
            propag(node);
            return;
        }
        propag(node);
        if (r < ql || l > qr) return;
        int mij = (l+r) / 2;
        update(2*node, l, mij);
        update(2*node+1, mij+1, r);
        for (int i = 0; i < 5; i++) {
            v[node].s[i] = (v[2*node].s[i] + v[2*node+1].s[i]) % MOD;
        }
    }
    void cuteUpdate(int ql, int qr, int qval)
    {
        this->ql = ql;
        this->qr = qr;
        getRecurrenceMatrix(qval, qMat, auxMat);
        update(1, 1, n2);
    }
    int query(int node, int l, int r)
    {
        propag(node);
        if (ql <= l && r <= qr) {
            return v[node].s[0];
        }
        int mij = (l+r) / 2;
        if (mij+1 > qr) {
            return query(2*node, l, mij);
        }
        if (mij < ql) {
            return query(2*node+1, mij+1, r);
        }
        return query(2*node, l, mij) + query(2*node+1, mij+1, r);
    }
    int cuteQuery(int ql, int qr)
    {
        this->ql = ql;
        this->qr = qr;
        return (query(1, 1, n2) % MOD + MOD) % MOD;
    }
} aint;

int main()
{
    ios_base::sync_with_stdio(false);
    cin.tie(NULL);
    cin >> n >> q;
    n2 = n;
    while (n2 & (n2-1)) {
        n2 += n2 & (-n2);
    }
    calcBinMats();
    for (i = 1; i <= n; i++) {
        cin >> a[i];
    }
    aint.init();
    for (i = 1; i <= q; i++) {
        cin >> currq.tip >> currq.l >> currq.r;
        if (currq.tip == 1) {
            cout << aint.cuteQuery(currq.l, currq.r) << '\n';
        }
        else {
            cin >> currq.val;
            aint.cuteUpdate(currq.l, currq.r, currq.val);
        }
    }
    return 0;
}
