#pragma GCC optimize("O3,unroll-loops")
#pragma GCC target("avx2")

#include <iostream>
#include <iomanip>

using namespace std;

#define BINMATNR 4
#define BINMATBITS 14
#define MOD 100000543LL

int n, n2, q, a[100010];

struct Query
{
    int tip, l, r;
    long long val;
} currq;

struct Matrix
{
    long long a[25];
    void zero()
    {
        for (int i = 0; i < 25; i++) {
            a[i] = 0;
        }
    }
    void identity()
    {
        zero();
        a[0] = a[6] = a[12] = a[18] = a[24] = 1;
    }
    static void cpy(Matrix &dst, const Matrix &src)
    {
        for (int i = 0; i < 25; i++) {
            dst.a[i] = src.a[i];
        }
    }
    static void mult(const Matrix &a, const Matrix &b, Matrix &c)
    {
        for (int i = 0; i < 5; i++) {
            for (int j = 0; j < 5; j++) {
                #define F(k) a.a[i*5+k] * b.a[k*5+j]
                c.a[i*5+j] = (F(0) + F(1) + F(2) + F(3) + F(4)) % MOD;
                #undef F
            }
        }
    }
    static void mult(const Matrix &a, long long *v, long long *w)
    {
        for (int i = 0; i < 5; i++) {
            #define F(k) a.a[i*5+k] * v[k]
            w[i] = (F(0) + F(1) + F(2) + F(3) + F(4)) % MOD;
            #undef F
        }
    }
};

Matrix binMatUp[BINMATNR][(1<<BINMATBITS)], binMatDown[BINMATNR][(1<<BINMATBITS)];
void calcBinMats()
{
    binMatUp[0][0].identity();
    binMatDown[0][0].identity();
    binMatUp[0][1] = {
    0, 1, 0, 0, 0,
    0, 0, 1, 0, 0,
    0, 0, 0, 1, 0,
    0, 0, 0, 0, 1,
    1, 2, 3, 4, 5
    };
    binMatDown[0][1] = {
    -2, -3, -4, -5, 1,
    1, 0, 0, 0, 0,
    0, 1, 0, 0, 0,
    0, 0, 1, 0, 0,
    0, 0, 0, 1, 0
    };
    for (int j = 2; j < (1 << BINMATBITS); j++) {
        Matrix::mult(binMatUp[0][j-1], binMatUp[0][1], binMatUp[0][j]);
        Matrix::mult(binMatDown[0][j-1], binMatDown[0][1], binMatDown[0][j]);
    }
    for (int i = 1; i < BINMATNR; i++) {
        binMatUp[i][0].identity();
        binMatDown[i][0].identity();
        Matrix::mult(binMatUp[i-1][(1<<BINMATBITS)-1], binMatUp[i-1][1], binMatUp[i][1]);
        Matrix::mult(binMatDown[i-1][(1<<BINMATBITS)-1], binMatDown[i-1][1], binMatDown[i][1]);
        for (int j = 2; j < (1 << BINMATBITS); j++) {
            Matrix::mult(binMatUp[i][j-1], binMatUp[i][1], binMatUp[i][j]);
            Matrix::mult(binMatDown[i][j-1], binMatDown[i][1], binMatDown[i][j]);
        }
    }
}
void getRecurrenceMatrix(long long step, Matrix &ans, Matrix &aux)
{
    ans.identity();
    if (step >= 0) {
        for (int i = 0; step; i++) {
            Matrix::mult(ans, binMatUp[i][step&((1<<BINMATBITS)-1)], aux);
            Matrix::cpy(ans, aux);
            step >>= BINMATBITS;
        }
    }
    else {
        step = -step;
        for (int i = 0; step; i++) {
            Matrix::mult(ans, binMatDown[i][step&((1<<BINMATBITS)-1)], aux);
            Matrix::cpy(ans, aux);
            step >>= BINMATBITS;
        }
    }
}

#define int long long

struct Aint
{
    struct Data
    {
        long long s[5];
        long long lazy;
    };
    Data v[262152];
    long long auxS[5];
    Matrix auxMat, auxMat2;
    int ql, qr;
    long long qval;
    void propag(int node)
    {
        if (!v[node].lazy) return;
        getRecurrenceMatrix(v[node].lazy, auxMat, auxMat2);
        Matrix::mult(auxMat, v[node].s, auxS);
        #define F(k) v[node].s[k] = auxS[k];
        F(0) F(1) F(2) F(3) F(4)
        #undef F
        if (node < n2) {
            v[2*node].lazy += v[node].lazy;
            v[2*node+1].lazy += v[node].lazy;
        }
        v[node].lazy = 0;
    }
    void init()
    {
        for (int i = n2; i < 2*n2; i++) {
            for (int j = 0; j < 5; j++) {
                v[i].s[j] = j;
            }
        }
        for (int i = 1; i <= n; i++) {
            v[n2+i-1].lazy = a[i];
            propag(n2+i-1);
        }
        for (int i = n2-1; i >= 1; i--) {
            for (int j = 0; j < 5; j++) {
                v[i].s[j] = v[2*i].s[j] + v[2*i+1].s[j];
            }
        }
    }
    void update(int node, int l, int r)
    {
        if (ql <= l && r <= qr) {
            v[node].lazy += qval;
            propag(node);
            return;
        }
        propag(node);
        if (r < ql || l > qr) return;
        int mij = (l+r) / 2;
        update(2*node, l, mij);
        update(2*node+1, mij+1, r);
        #define F(k) v[node].s[k] = (v[2*node].s[k] + v[2*node+1].s[k]) % MOD;
        F(0) F(1) F(2) F(3) F(4)
        #undef F
    }
    void cuteUpdate(int ql, int qr, long long qval)
    {
        this->ql = ql;
        this->qr = qr;
        this->qval = qval;
        update(1, 1, n2);
    }
    int query(int node, int l, int r)
    {
        propag(node);
        if (ql <= l && r <= qr) {
            return v[node].s[0];
        }
        int mij = (l+r) / 2;
        if (mij+1 > qr) {
            return query(2*node, l, mij);
        }
        if (mij < ql) {
            return query(2*node+1, mij+1, r);
        }
        return query(2*node, l, mij) + query(2*node+1, mij+1, r);
    }
    int cuteQuery(int ql, int qr)
    {
        this->ql = ql;
        this->qr = qr;
        return (query(1, 1, n2) % MOD + MOD) % MOD;
    }
} aint;

signed main()
{
    ios_base::sync_with_stdio(false);
    cin.tie(NULL);
    cin >> n >> q;
    n2 = n;
    while (n2 & (n2-1)) {
        n2 += n2 & (-n2);
    }
    calcBinMats();
    for (int i = 1; i <= n; i++) {
        cin >> a[i];
    }
    aint.init();
    for (int i = 1; i <= q; i++) {
        cin >> currq.tip >> currq.l >> currq.r;
        if (currq.tip == 1) {
            cout << aint.cuteQuery(currq.l, currq.r) << '\n';
        }
        else {
            cin >> currq.val;
            aint.cuteUpdate(currq.l, currq.r, currq.val);
        }
    }
    return 0;
}
