#include <bits/stdc++.h>

using namespace std;
typedef long long ll;

const int N_MAX = 100000;
const int Q_MAX = 200000;
const int MOD = (int) 1e8 + 543;

const int BUFFER_SIZE = 16384;
int bpos = BUFFER_SIZE - 1;
char buffer[BUFFER_SIZE];

char read_char () {
    if (++bpos == BUFFER_SIZE) {
        fread(buffer, sizeof(char), BUFFER_SIZE, stdin);
        bpos = 0;
    }
    return buffer[bpos];
}
int read_int () {
    int val = 0; char c;
    while (!isdigit(c = read_char()) && c != '-');
    bool sgn = false;
    if (c == '-') {
        sgn = true;
        c = read_char();
    }
    do {
        val = val * 10 + (int) (c - '0');
    } while (isdigit(c = read_char()));
    if (sgn == true) {
        val = -val;
    }
    return val;
}

struct Matrix {
    int val[5][5];
};
Matrix operator * (const Matrix &A, const Matrix &B) {
    Matrix C;
    for (int i = 0; i < 5; i++) {
        for (int j = 0; j < 5; j++) {
            ll sum = 0;
            for (int k = 0; k < 5; k++) {
                sum += (ll) A.val[i][k] * B.val[k][j];
            }
            C.val[i][j] = sum % MOD;
        }
    }
    return C;
}
Matrix operator + (const Matrix &A, const Matrix &B) {
    Matrix C;
    for (int i = 0; i < 5; i++) {
        for (int j = 0; j < 5; j++) {
            C.val[i][j] = A.val[i][j] + B.val[i][j];
            C.val[i][j] -= (C.val[i][j] >= MOD) * MOD;
        }
    }
    return C;
}
Matrix R, Ri, I, O;
Matrix pwr (const Matrix &A, const ll &e) {
    if (e == 1) {
        return A;
    } else if (e & 1) {
        return A * pwr(A, (e ^ 1));
    } else {
        Matrix aux = pwr(A, (e >> 1));
        return aux * aux;
    }
}

void print (const Matrix &A) {
    for (int i = 0; i < 5; i++) {
        for (int j = 0; j < 5; j++) {
            cout << A.val[i][j] << " ";
        }
        cout << "\n";
    }
    cout << "\n";
}

const int LOG = 60;
Matrix pR[LOG], pRi[LOG];

Matrix get (const ll &e) {
    Matrix ret = I;
    for (int i = 0; i < LOG; i++) {
        if ((e >> i) & 1) {
            ret = ret * pR[i];
        }
    }
    return ret;
}
Matrix geti (const ll &e) {
    Matrix ret = I;
    for (int i = 0; i < LOG; i++) {
        if ((e >> i) & 1) {
            ret = ret * pRi[i];
        }
    }
    return ret;
}

void init () {
    vector <vector <int>> Rv = {
        {0, 1, 0, 0, 0},
        {0, 0, 1, 0, 0},
        {0, 0, 0, 1, 0},
        {0, 0, 0, 0, 1},
        {1, 2, 3, 4, 5}
    };
    vector <vector <int>> Riv = {
        {-2 + MOD, -3 + MOD, -4 + MOD, -5 + MOD, 1},
        {1, 0, 0, 0, 0},
        {0, 1, 0, 0, 0},
        {0, 0, 1, 0, 0},
        {0, 0, 0, 1, 0}
    };
    for (int i = 0; i < 5; i++) {
        for (int j = 0; j < 5; j++) {
            R.val[i][j] = Rv[i][j];
            Ri.val[i][j] = Riv[i][j];
        }
    }
    I = R * Ri;
    pR[0] = R; pRi[0] = Ri;
    for (int i = 1; i < LOG; i++) {
        pR[i] = pR[i - 1] * pR[i - 1];
        pRi[i] = pRi[i - 1] * pRi[i - 1];
    }
}

Matrix f (const ll &k) {
    return (k == 0 ? I : (k > 0 ? get(k) : geti(-k)));
}

int n, q;
ll a[N_MAX + 2];

struct SegInfo {
    Matrix sum;
    Matrix lazy;
    bool has_lazy;
    SegInfo () {
        sum = O; lazy = I;
        has_lazy = false;
    }
    SegInfo (const Matrix &_sum, const Matrix &_lazy) {
        sum = _sum; lazy = _lazy; has_lazy = false;
    }
    void mul (const Matrix &A) {
        sum = sum * A;
        lazy = lazy * A;
        has_lazy = true;
    }
};
SegInfo operator + (const SegInfo &L, const SegInfo &R) {
    return SegInfo(L.sum + R.sum, I);
}

SegInfo seg_tree[N_MAX * 4 + 2];

void build (int node, int l, int r) {
    if (l == r) {
        seg_tree[node] = SegInfo(f(a[l] - 4), I);
        return;
    }
    int mid = (l + r) / 2;
    int left = node * 2, right = node * 2 + 1;
    build(left, l, mid);
    build(right, mid + 1, r);
    seg_tree[node] = seg_tree[left] + seg_tree[right];
}
void build () {
    build(1, 1, n);
}

void update (int node, int l, int r, int ul, int ur, const Matrix &U) {
    if (ul <= l && r <= ur) {
        seg_tree[node].mul(U);
        return;
    }
    int mid = (l + r) / 2;
    int left = node * 2, right = node * 2 + 1;
    if (seg_tree[node].has_lazy == true) {
        seg_tree[left].mul(seg_tree[node].lazy);
        seg_tree[right].mul(seg_tree[node].lazy);
    }
    if (ul <= mid) {
        update(left, l, mid, ul, ur, U);
    }
    if (mid + 1 <= ur) {
        update(right, mid + 1, r, ul, ur, U);
    }
    seg_tree[node] = seg_tree[left] + seg_tree[right];
}
void update (int ul, int ur, int val) {
    update(1, 1, n, ul, ur, f(val));
}

Matrix query (int node, int l, int r, int ql, int qr) {
    if (ql <= l && r <= qr) {
        return seg_tree[node].sum;
    }
    int mid = (l + r) / 2;
    int left = node * 2, right = node * 2 + 1;
    if (seg_tree[node].has_lazy == true) {
        seg_tree[left].mul(seg_tree[node].lazy);
        seg_tree[right].mul(seg_tree[node].lazy);
        seg_tree[node].lazy = I;
        seg_tree[node].has_lazy = false;
    }
    Matrix sum = O;
    if (ql <= mid) {
        sum = query(left, l, mid, ql, qr);
    }
    if (mid + 1 <= qr) {
        sum = sum + query(right, mid + 1, r, ql, qr);
    }
    return sum;
}
Matrix query (int ql, int qr) {
    return query(1, 1, n, ql, qr);
}

int main () {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    mt19937 rnd(0);
    init();
    n = read_int();
    q = read_int();
//    n = N_MAX; q = Q_MAX;
    for (int i = 1; i <= n; i++) {
        a[i] = read_int();
//        a[i] = rnd() % (MOD * 2 - 1) - (MOD - 1);
    }
    build();
    while (q--) {
        int type = read_int();
//        int type = rnd() % 2 + 1;
        if (type == 2) {
            int l = read_int(), r = read_int(), val = read_int();
//            int l = rnd() % n + 1, r = rnd() % n + 1;
//            r = l;
//            if (l > r) swap(l, r);
//            int val = rnd() % (MOD * 2 - 1) - (MOD - 1);
//            for (int i = l; i <= r; i++) {
//                a[i] += val;
//            }
            update(l, r, val);
        } else {
            int l = read_int(), r = read_int();
//            int l = rnd() % n + 1, r = rnd() % n + 1;
//            r = l;
//            if (l > r) swap(l, r);
            Matrix sum = query(l, r);
//            Matrix sum = O;
//            for (int i = l; i <= r; i++) {
//                sum = sum + f(a[i] - 4);
//            }
            int answer = 0;
            for (int i = 0; i < 5; i++) {
                answer = (answer + i * sum.val[4][i]) % MOD;
            }
            cout << answer << "\n";
        }
    }

    return 0;
}
