#include <bits/stdc++.h>

using namespace std;

const int M = 1e8 + 543;

template<class T>
T pow(T v, int64_t e) {
    T re(1);
    for(int i = 0; (1ll << i) <= e; ++i) {
        if(e & (1ll << i)) re = re * v;
        v = v * v;
    }
    return re;
}

struct ZP {
    ZP(int64_t v = 0) : x((v % M + M) % M) {}

    ZP &operator+=(const ZP &rhs) {
        if((x += rhs.x) >= M)
            x -= M;
        return *this;
    }

    ZP &operator-=(const ZP &rhs) {
        if((x -= rhs.x) < 0) x += M;
        return *this;
    }

    ZP operator+(const ZP &rhs) { return ZP(*this) += rhs; }
    ZP operator-(const ZP &rhs) { return ZP(*this) -= rhs; }

    ZP operator*(const ZP &rhs) const { return ZP(1ll * x * rhs.x); }

    ZP &operator*=(const ZP &rhs) {
        *this = *this * rhs;
        return *this;
    }

    ZP inv() const { return pow(*this, M - 2); }

    ZP operator/(const ZP &rhs) { return ZP(*this) * rhs.inv(); }
    ZP &operator/=(const ZP &rhs) {
        *this = *this / rhs;
        return *this;
    }

    friend ostream &operator<<(ostream &o, ZP that) {
        o << that.x;
        return o;
    }

    int x;
};

using ll = long long;

struct matr {
    array<array<ll, 5>, 5> V;
    matr(int v = 0) {
        for(int i = 0; i < 5; ++i)
            for(int j = 0; j < 5; ++j)
                V[i][j] = 0;
        if(v == 1) {
            for(int i = 0; i < 5; ++i) V[i][i] = 1;
        }
    }
    matr operator*(const matr &rhs) {
        matr re;
        for(int i = 0; i < 5; ++i)
            for(int j = 0; j < 5; ++j)
                for(int k = 0; k < 5; ++k)
                    re.V[i][k] += V[i][j] * rhs.V[j][k];
        for(int i = 0; i < 5; ++i)
            for(int j = 0; j < 5; ++j)
                re.V[i][j] %= M;
        return re;
    }
    matr &operator*=(const matr &rhs) {
        *this = *this * rhs;
        return *this;
    }
};

struct vec {
    array<ll, 5> V;

    vec &operator+=(const vec &rhs) {
        for(int i = 0; i < 5; ++i) {
            V[i] += rhs.V[i];
            if(V[i] >= M) V[i] -= M;
        }
        return *this;
    }

    vec &operator-=(const vec &rhs) {
        for(int i = 0; i < 5; ++i) {
            V[i] -= rhs.V[i];
            if(V[i] < 0) V[i] += M;
        }
        return *this;
    }

    vec operator+(const vec &rhs) { return vec(*this) += rhs; }
    vec operator-(const vec &rhs) { return vec(*this) -= rhs; }

    friend vec operator*(const matr &rhs, vec that) {
        vec re;
        for(int i = 0; i < 5; ++i)
            for(int j = 0; j < 5; ++j)
                re.V[i] += rhs.V[i][j] * that.V[j];
        for(int i = 0; i < 5; ++i)
            re.V[i] %= M;
        return re;
    }
};

matr A, InvA;
vec Vec0{4, 3, 2, 1, 0};

struct AINT {
    int n;

    vector<vec> S;
    vector<matr> Lz;

    AINT(int N) : n(N), S(4 * N), Lz(4 * N, matr(1)) {}

    void prop(int u, int s, int d) {
    //    S[u] = Lz[u] * S[u];
    //    if(s != d) {
    //        Lz[u << 1] = Lz[u] * Lz[u << 1];
    //        Lz[u << 1 | 1] = Lz[u] * Lz[u << 1 | 1];
    //    }
    //    Lz[u] = matr(1);
    }

    void update(int p, vec v, int u, int s, int d) {
        prop(u, s, d);
        if(d < p || p < s) return;
        if(s == d) {
            S[u] = v;
            return;
        }
        update(p, v, u << 1, s, (s + d) >> 1);
        update(p, v, u << 1 | 1, ((s + d) >> 1) + 1, d);
        S[u] = S[u << 1] + S[u << 1 | 1];
        S[u] = Lz[u] * S[u];
    }

    void update(int p, vec v) { update(p, v, 1, 0, n - 1); }

    void update(int l, int r, matr V, int u, int s, int d) {
        prop(u, s, d);
        if(r < s || d < l) return;
        if(l <= s && d <= r) {
            Lz[u] *= V;
            S[u] = V * S[u];
            return;
        }
        update(l, r, V, u << 1, s, (s + d) >> 1);
        update(l, r, V, u << 1 | 1, ((s + d) >> 1) + 1, d);
        S[u] = S[u << 1] + S[u << 1 | 1];
        S[u] = Lz[u] * S[u];
    }

    void update(int l, int r, matr V) { update(l, r, V, 1, 0, n - 1); }

    vec query(int l, int r, int u, int s, int d) {
        prop(u, s, d);
        if(r < s || d < l) return vec();
        if(l <= s && d <= r)
            return S[u];
        return Lz[u] * (query(l, r, u << 1, s, (s + d) >> 1) 
            + query(l, r, u << 1 | 1, ((s + d) >> 1) + 1, d));
    }

    ll query(int l, int r) { return query(l, r, 1, 0, n - 1).V[4]; }
};

int main() {
    cin.tie(0);
    ios_base::sync_with_stdio(0);

    A.V[0][0] = 5; A.V[0][1] = 4; A.V[0][2] = 3; A.V[0][3] = 2; A.V[0][4] = 1;
    A.V[1][0] = 1; A.V[2][1] = 1; A.V[3][2] = 1; A.V[4][3] = 1;
    InvA.V[0][1] = 1; InvA.V[1][2] = 1; InvA.V[2][3] = 1; InvA.V[3][4] = 1;
    InvA.V[4][0] = 1; InvA.V[4][1] = M-5; InvA.V[4][2] = M-4; InvA.V[4][3] = M-3; InvA.V[4][4] = M-2;
    
    int n, q;
    cin >> n >> q;

    vector<matr> P2A = {A}, P2InvA = {InvA}; 
    while((1 << (int(P2A.size()) - 1)) < M) P2A.push_back(P2A.back() * P2A.back());
    while((1 << (int(P2InvA.size()) - 1)) < M) P2InvA.push_back(P2InvA.back() * P2InvA.back());
    auto getm = [&](int a) -> matr {
        if(a >= 0) {
            matr re(1);
            for(int i = 0; (1 << i) <= a; ++i)
                if(a & (1 << i)) re = re * P2A[i];
            return re;
        }
        a = -a;
        matr re(1);
        for(int i = 0; (1 << i) <= a; ++i)
            if(a & (1 << i)) re = re * P2InvA[i];
        return re;
    };
    AINT Sol(n);

    for(int i = 0; i < n; ++i) {
        int a;
        cin >> a;
        Sol.update(i, getm(a) * Vec0);
    }

    for(int i = 0; i < q; ++i) {
        int tip, l, r;
        cin >> tip >> l >> r;
        --l; --r;
        if(tip == 1) {
            cout << Sol.query(l, r) << "\n";
        } else {
            int a;
            cin >> a;
            matr M = getm(a);
            Sol.update(l, r, M);
        }
    }
    return 0;
}
