#include <bits/stdc++.h>
using namespace std;
double timePassed();


constexpr int EMPTY = 0;
constexpr int HOR_SINGLE = -1;
constexpr int VERT_SINGLE = -2;
constexpr int HOR_DOUBLE = -3;
constexpr int VERT_DOUBLE = -4;

const int N = 201;
const int MSZ = 47;

double current_tl;
double totalTL;
const double dt = 1e-3;
double last_completed;
double start_time;
int subsets_remain;

void init(int numSubtests, double timeLimit) {
    totalTL = timeLimit;
    subsets_remain = numSubtests;
    start_time = timePassed();
}
int degree[N] = {};
int x[N];
int y[N];
pair<int, int> edges[8 * N];
int edgeN;
int lastInRow[MSZ], lastInCol[MSZ];
int cnt[MSZ][MSZ];
int idx[8 * N][8 * N];
int eptr = 0;
int dptr = 0;

int sz[N];
int graph[N][8];

void drawVertical(int v, int u, vector<vector<int>>& puzzle) {
    int S = x[v];
    int E = x[u];
    const int Y = y[v];
    if (S > E) swap(S, E);
    for (int x1 = S + 1; x1 < E; ++x1) puzzle[x1][Y] = (puzzle[x1][Y] == 0 ? VERT_SINGLE : VERT_DOUBLE);
}
void drawHorizontal(int v, int u, vector<vector<int>>& puzzle) {
    int S = y[v];
    int E = y[u];
    const int X = x[v];
    if (S > E) swap(S, E);
    for (int y1 = S + 1; y1 < E; ++y1) puzzle[X][y1] = (puzzle[X][y1] == 0 ? HOR_SINGLE : HOR_DOUBLE);
}
void addVertical(int v, int u, vector<vector<int>>& puzzle, int d = 1) {
    int S = x[v];
    int E = x[u];
    const int Y = y[v];
    if (S > E) swap(S, E);
    for (int x1 = S + 1; x1 < E; ++x1) puzzle[x1][Y] += d;
    return;
}
bool checkHorizontal(int v, int u, vector<vector<int>>& puzzle) {
    int S = y[v];
    int E = y[u];
    const int X = x[v];
    if (S > E) swap(S, E);
    for (int y1 = S + 1; y1 < E; ++y1) {
        if (puzzle[X][y1] != 0) return false;
    }
    return true;
}

bool solve(std::vector<std::vector<int>>& puzzle) {
    srand(time(0));
    current_tl = (totalTL - start_time);
    for (int i = 0; i < MSZ; ++i) {
        lastInCol[i] = -1;
        lastInRow[i] = -1;
    }
    memset(sz, 0, sizeof(sz));
    eptr = edgeN = dptr = 0;
    const int n = puzzle.size();
    const int m = puzzle[0].size();
    for (int i = 0; i < n; ++i) memset(cnt[i], 0, sizeof(cnt[i]));
    int need = 0;
    for (int i = 0; i < N; ++i)
        memset(graph[i], -1, sizeof(graph[i]));
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < m; ++j) if (puzzle[i][j] != 0) {
            int lc = lastInCol[j];
            int lr = lastInRow[i];
            if (lc != -1) {
                edgeN += 2;

                graph[lc][sz[lc]] = dptr;
                idx[lc][dptr] = eptr;
                edges[eptr] = {lc, dptr};
                ++eptr; 
                ++sz[lc];
                
                graph[dptr][sz[dptr]] = lc;
                idx[dptr][lc] = eptr;
                edges[eptr] = {dptr, lc};
                ++eptr;
                ++sz[dptr];
            }
            if (lr != -1) {
                edgeN += 2;

                graph[lr][sz[lr]] = dptr;
                idx[lr][dptr] = eptr;
                edges[eptr] = {lr, dptr};
                ++eptr; 
                ++sz[lr];
                
                graph[dptr][sz[dptr]] = lr;
                idx[dptr][lr] = eptr;
                edges[eptr] = {dptr, lr};
                ++eptr;
                ++sz[dptr];
            }
            need += degree[dptr];
            degree[dptr] = puzzle[i][j];
            lastInCol[j] = dptr;
            lastInRow[i] = dptr;
            x[dptr] = i;
            y[dptr] = j;
            ++dptr;
        }
    }
    need /= 2;
    --subsets_remain;
    last_completed = timePassed();

    // for (int i = 0; i < dptr; ++i) {
    //     cout << x[i] << " " << y[i] << " " << sz[i] <<": ";
    //     for (int j = 0; j < sz[i]; ++j) {
    //         cout << graph[i][j] << ", ";
    //     }
    //     cout << "\n";
    // }

    int order[dptr] = {};
    iota(order, order + dptr, 0);
    int currentDegree[dptr] = {};
    bool incl[edgeN] = {};
    int degree_mismatch = 0;
    int tries = 0;
    // sort(order, order + dptr, [&] (int a, int b) {
    //     return degree[a] < degree[b];
    // });
    while (timePassed() + dt - start_time < current_tl) {
        random_shuffle(order, order + dptr);
        memset(currentDegree, 0, sizeof(currentDegree));
        vector<int> added;
        int ok = 0;    
        for (int i = 0; i < dptr; ++i) {
            int v = order[i];
            int need = degree[v] - currentDegree[v];
            random_shuffle(graph[v], graph[v] + sz[v]);
            for (int j = 0; j < sz[v]; ++j) {
                int u = graph[v][j];
                if (currentDegree[u] == degree[u]) continue;
                if (currentDegree[v] == degree[v]) break;
                currentDegree[u]++;
                currentDegree[v]++;
                added.push_back(idx[v][u]);
            }
        }
        for (int i = 0; i < dptr; ++i) {
            if (currentDegree[i] == degree[i]) {
                ++ok;
            }
        }
        if (ok != dptr) continue;
        // if (ok == dptr) {
        //     cerr << "PROGRESS!!!\n";
        //     return false;
        // }    
        for (int i = 0; i < added.size(); ++i) {
            auto [v, u] = edges[added[i]];
            if (x[v] == x[u]) {
                addVertical(v, u, puzzle, 1);
            }
        }
        bool bad = false;
        for (int i = 0; i < added.size(); ++i) {
            auto [v, u] = edges[added[i]];
            if (y[v] == y[u]) {
                bad |= !checkHorizontal(v, u, puzzle);
            }
        }
        for (int i = 0; i < added.size(); ++i) {
            auto [v, u] = edges[added[i]];
            if (x[v] == x[u]) {
                addVertical(v, u, puzzle, -1);
            }
        }
        if (bad) continue;
        for (int i = 0; i < added.size(); ++i) {
            auto [v, u] = edges[added[i]];
            if (y[v] == y[u]) {
                drawVertical(v, u, puzzle);
            }
            else {
                drawHorizontal(v, u, puzzle);
            }
        }
        // for (int i = 0; i < n; ++i) {
        //     for (int j = 0; j < m; ++j) {
        //         cout << puzzle[i][j] << " ";
        //     }
        //     cout << "\n";
        // }
        return true;
    }
    return false;
}

